#ifndef _AI_DRIVER_H_
#define _AI_DRIVER_H_

#include "ai_utils.h"
#include "PathNode.h"
#include <vector>

class AI_Driver
{
public:

	enum Flying_State
	{
		FLYING_STATE_GOAL_SEARCHING,
		FLYING_STATE_BACK_TRACKING,
		FLYING_STATE_RETURN_TO_NODE,
		FLYING_STATE_LANDING,
		FLYING_STATE_LANDED
	};

	AI_Driver();
	~AI_Driver();

	void update_data(float distance_to_goal, float lander_angle, float sensors[8], float goal_x, float goal_y, 
					 float lander_x, float lander_y, float lander_x_vel, float lander_y_vel, float fuel_left);

	void update(float dT);

	void output_debug_data();

	bool turning_left();
	bool turning_right();
	bool thrusters_on();

private:

	void delete_path_nodes();

	float get_closest_sensor_distance();
	bool is_point_within_path_nodes(Vector2 loc);
	bool is_point_close_to_any_node(Vector2 loc);

	void handle_path_finding(float dT);
	void handle_flying(float dT);

	void calculate_next_valid_path_on_node(Path_Node* node);

	void handle_goal_search_flying_old(float dT);
	void handle_goal_search_flying(float dT);
	void handle_landing(float dT);

	float m_distance_to_goal;
	float m_angle_to_goal;

	float m_lander_angle;
	float m_sensor_data[8];

	Vector2 m_position;
	Vector2 m_previous_position;
	Vector2 m_goal_position;
	Vector2 m_velocity;

	Vector2 m_target_location;
	float m_target_angle;
	float m_angle_to_target;

	bool m_back_tracking;

	float m_time_taken_to_travel_to_next_waypoint;
	float m_fuel_left;

	bool m_turn_left;
	bool m_turn_right;
	bool m_thrusters_on;

	std::vector<Path_Node*> m_path_nodes;

	Path_Node* m_current_path_node;

	Flying_State m_flying_state;
};

#endif