#ifndef _PATH_NODE_H_
#define _PATH_NODE_H_

#include "ai_utils.h"

class Path_Node
{
public:

	Path_Node();
	Path_Node(Vector2 center, float radius, Vector2 goal_position, float angle_to_goal, float sensor_data[8], Path_Node* spawned_from);
	void calculate_valid_paths();
	void calculate_path_to_pursue();
	void invalidate_current_path();

	void draw();

	float get_radius();
	Vector2 get_center();
	bool is_within_node(Vector2 pos);

	int get_num_potential_paths();
	Path_Node* get_node_spawned_from();
	Vector2 get_suggested_next_waypoint();
	bool is_evaluating_long_path();

private:

	// This number is 16 because we are considering a long path and short path in each direction.
	static const int MAX_POTENTIAL_PATHS = MAX_SENSORS;

	void invalidate_path_arrived_from();

	Vector2 m_center;
	float m_radius;

	int m_current_path;
	Vector2 m_goal_position;
	float m_angle_to_goal;

	int m_potiential_short_paths_count;
	bool m_potential_short_paths[MAX_POTENTIAL_PATHS];

	int m_potiential_long_paths_count;
	bool m_potential_long_paths[MAX_POTENTIAL_PATHS];

	bool m_evaluating_long_path;

	float m_sensor_data[MAX_SENSORS];
	Path_Node* m_node_spawned_from;
	Vector2 m_next_suggested_waypoint;
};

#endif