#include "ai_utils.h"

namespace Utils
{

	float calc_angle(Vector2 a, Vector2 b);
	float squared_distance_between(Vector2 a, Vector2 b);
	float distance_between(Vector2 a, Vector2 b);

	float calc_angle(Vector2 a, Vector2 b)
	{
		float angle = (float)atan2(b.Y - a.Y, b.X - a.X);
		return angle < 0? angle + (float)PI * 2.0f : angle;
	}

	float squared_distance_between(Vector2 a, Vector2 b)
	{
		float x = a.X - b.X;
		float y = a.Y - b.Y;

		return x * x + y * y;
	}

	float distance_between(Vector2 a, Vector2 b)
	{
		float x = a.X - b.X;
		float y = a.Y - b.Y;

		return sqrt(x * x + y * y);
	}

	float magnitude(Vector2 a)
	{
		return sqrt(a.X * a.X + a.Y * a.Y);
	}

	float max(float a, float b)
	{
		if(a > b)
			return a;
		return b;
	}

	float min(float a, float b)
	{
		if(a < b)
			return a;
		return b;
	}

	bool direction_is_upward(int direction)
	{
		return direction <= 2 && direction >= 0;
	}

	bool direction_is_downward(int direction)
	{
		return direction >= 5;
	}

	void draw_circle(Vector2 position, float radius, unsigned __int32 color)
	{
		const float LINES_IN_CIRCLE = 16;

		for(int i = 0; i < LINES_IN_CIRCLE; ++i)
		{
			float start_angle = 2 * PI * (i / LINES_IN_CIRCLE);
			float end_angle = 2 * PI * ((i + 1) / LINES_IN_CIRCLE);

			Lunar_AI::Draw_Line(position.X + radius * cos(start_angle), position.Y + radius * sin(start_angle), position.X + radius * cos(end_angle), position.Y + radius * sin(end_angle), false, color);
		}
	}

	void draw_line(Vector2 a, Vector2 b, unsigned __int32 color)
	{
		Lunar_AI::Draw_Line(a.X, a.Y, b.X, b.Y, false, color);
	}

}