#ifndef _AI_UTILS_H_
#define _AI_UTILS_H_

#include <cmath>

#define PI 3.14159f

#define RED 0xFFFF0000
#define GREEN 0xFF00FF00
#define BLUE 0xFF0000FF
#define ORANGE 0xFFFF7F00

#define TRANSPARENT_BLUE 0x770000FF

#define MAX_SENSORS 8

#define NORTH_WEST_DIRECTION 0
#define NOTH_DIRECTION 1
#define NORTH_EAST_DIRECTION 2
#define WEST_DIRECTION 3
#define EAST_DIRECTION 4
#define SOUTH_WEST_DIRECTION 5
#define SOUTH_DIRECTION 6
#define SOUTH_EAST_DIRECTION 7

#define NORTH_WEST_SENSOR m_sensor_data[0]
#define NORTH_SENSOR m_sensor_data[1]
#define NORTH_EAST_SENSOR m_sensor_data[2]
#define WEST_SENSOR m_sensor_data[3]
#define EAST_SENSOR m_sensor_data[4]
#define SOUTH_WEST_SENSOR m_sensor_data[5]
#define SOUTH_SENSOR m_sensor_data[6]
#define SOUTH_EAST_SENSOR m_sensor_data[7]

#define ROTATION_SPEED PI / 100
#define GRAVITY -1.63
#define THRUST_ACCELERATION 1.63 * 5

struct Vector2
{
	float X;
	float Y;

	Vector2()
	{
		X = 0;
		Y = 0;
	}

	Vector2(float x, float y)
	{
		X = x;
		Y = y;
	}

	bool operator == (Vector2 rhs)
	{
		return rhs.X == X && rhs.Y == Y;
		//return abs(rhs.X - X) < 0.00025f && abs(rhs.Y - Y) < 0.00025f;
	}
};

const float COS_0 = 1;
const float SIN_0 = 0;
const float COS_PI_4 = cos(PI / 4.0f);
const float SIN_PI_4 = sin(PI / 4.0f);
const float COS_PI_2 = cos(PI / 2.0f);
const float SIN_PI_2 = sin(PI / 2.0f);
const float COS_3_PI_4 = cos(3 * PI / 4.0f);
const float SIN_3_PI_4 = sin(3 * PI / 4.0f);
const float COS_PI = cos(PI);
const float SIN_PI = sin(PI);
const float COS_5_PI_4 = cos(5 * PI / 4.0f);
const float SIN_5_PI_4 = sin(5 * PI / 4.0f);
const float COS_3_PI_2 = cos(3 * PI / 2.0f);
const float SIN_3_PI_2 = sin(3 * PI / 2.0f);
const float COS_7_PI_4 = cos(7 * PI / 4.0f);
const float SIN_7_PI_4 = sin(7 * PI / 4.0f);

const float sensor_angles[8] = {3 * PI / 4, PI / 2, PI / 4, PI, 0, 5 * PI / 4, 3 * PI / 2, 7 * PI / 4};
const float cos_of_sensor_angles[8] = {COS_3_PI_4, COS_PI_2, COS_PI_4, COS_PI, COS_0, COS_5_PI_4, COS_3_PI_2, COS_7_PI_4};
const float sin_of_sensor_angles[8] = {SIN_3_PI_4, SIN_PI_2, SIN_PI_4, SIN_PI, SIN_0, SIN_5_PI_4, SIN_3_PI_2, SIN_7_PI_4};

namespace Lunar_AI
{
	void Draw_Text( float a_x, float a_y, float a_scale_multiplier, bool a_screen_coords, unsigned __int32 a_color, const char * a_format, ... );
	void Draw_Line( float a_x1, float a_y1, float a_x2, float a_y2, bool a_screen_coords, unsigned __int32 a_color );
}

namespace Utils
{

	float calc_angle(Vector2 a, Vector2 b);
	float squared_distance_between(Vector2 a, Vector2 b);
	float distance_between(Vector2 a, Vector2 b);
	float magnitude(Vector2 a);
	float max(float a, float b);
	float min(float a, float b);

	bool direction_is_upward(int direction);
	bool direction_is_downward(int direction);

	bool direction_is_leftward(int direction);
	bool direction_is_rightward(int direction);

	void draw_circle(Vector2 position, float radius, unsigned __int32 color);
	void draw_line(Vector2 a, Vector2 b, unsigned __int32 color);
}

#endif